
# q-expansion of newform 448.2.a.j, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 448
weight = 2
poly_data = [-1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [1, 1], [-1, 1], [1, 0], [2, -2], [-3, -1], [0, -2], [-1, -1], [4, 0], [0, -2], [2, 2], [0, -2], [-4, 2], [-2, 2], [6, -2], [10, 0], [7, -1], [-9, 1], [-4, 0], [4, 4], [6, 4], [4, -4], [7, -1], [-6, 0], [8, -2], [-5, -3], [-10, -2], [0, 4], [-4, 2], [-8, 2], [0, -4], [-5, 3], [6, -4], [1, 1], [6, 4], [0, 4], [-3, -1], [-10, -6], [10, -6], [1, 3], [16, 4], [1, -5], [-4, -4], [-4, -2], [-2, 4], [2, -6], [12, 0], [4, -4], [15, -1], [1, 11], [2, 8], [4, 8], [8, -2], [-11, -3], [-14, 0], [20, 4], [-7, -5], [-24, 0], [10, 8], [26, 0], [-5, -5], [15, 1], [-13, -5], [8, 0], [-4, 2], [-18, -4], [-18, 2], [-12, -10], [-2, 2], [-11, 7], [2, 8], [-8, 4], [4, 12], [6, 4], [18, 6], [-2, 6], [0, -2], [13, -1], [-28, -2], [-8, 6], [-11, 5], [2, 0], [-28, 0], [-4, -6], [-32, 0], [-4, -8], [10, 8], [-12, -2], [-15, 3], [0, 16], [3, -13], [-22, -6], [20, 0], [-20, 8], [-4, 8], [4, 4], [21, -9], [-20, -6], [11, 3], [14, 4], [18, 6], [-18, -4], [29, 5], [4, -10], [10, -2], [-6, 0], [-3, -3], [6, -12], [0, -8], [-6, -16], [-8, -8], [-16, 6], [12, -10], [19, -13], [-36, 4], [28, 6], [37, -3], [2, 10], [-4, -6], [-6, 14], [7, -7], [6, 4], [-31, -5], [12, 16], [3, -5], [28, -6], [-24, -10], [2, -22], [-10, -10], [7, 1], [-10, -6], [16, -4], [36, 0], [-8, 14], [-8, -2], [4, 10], [-29, 5], [19, 3], [41, -5], [-4, 6], [15, -17], [-22, -8], [16, 16], [-24, -4], [-9, 1], [-26, -2], [-7, 3], [-32, -2], [45, 5], [-12, 4], [0, 6], [-2, -12], [-32, -4], [-2, 14], [8, 4], [-8, 12], [0, 16], [-16, 14], [-18, -4], [11, -11], [18, 6], [34, -8], [12, -8], [-25, -1], [14, 12], [-10, -2], [-24, 0], [11, 13]]
